@@ -10,7 +10,6 @@ Module implementing a previewer widget for HTML, Markdown and ReST files.
 from __future__ import unicode_literals
 
 try:  # Only for Py2
-    basestring
     import StringIO as io   # __IGNORE_EXCEPTION__
 except (ImportError, NameError):
     import io       # __IGNORE_WARNING__
@@ -23,8 +22,9 @@ import tempfile
 import sys
 
 from PyQt5.QtCore import pyqtSlot, pyqtSignal, Qt, QUrl, QSize, QThread
+from PyQt5.QtGui import QCursor
 from PyQt5.QtWidgets import QWidget, QVBoxLayout, QLabel, QCheckBox, \
-    QSizePolicy
+    QSizePolicy, QToolTip
 
 from E5Gui.E5Application import e5App
 
@@ -54,6 +54,7 @@ class PreviewerHTML(QWidget):
         try:
             from PyQt5.QtWebEngineWidgets import QWebEngineView
             self.previewView = QWebEngineView(self)
+            self.previewView.page().linkHovered.connect(self.__showLink)
             self.__usesWebKit = False
         except ImportError:
             from PyQt5.QtWebKitWidgets import QWebPage, QWebView
@@ -144,6 +145,16 @@ class PreviewerHTML(QWidget):
         Preferences.setUI("ShowFilePreviewSSI", checked)
         self.processEditor()
     
+    @pyqtSlot(str)
+    def __showLink(self, urlStr):
+        """
+        Private slot to show the hovered link in a tooltip.
+        
+        @param urlStr hovered URL
+        @type str
+        """
+        QToolTip.showText(QCursor.pos(), urlStr, self.previewView)
+    
     def processEditor(self, editor=None):
         """
         Public slot to process an editor's text.
@@ -192,15 +203,22 @@ class PreviewerHTML(QWidget):
                 self.__processingThread.process(
                     fn, language, editor.text(),
                     self.ssiCheckBox.isChecked(), rootPath,
-                    Preferences.getEditor("PreviewRestUseSphinx"))
+                    Preferences.getEditor("PreviewRestUseSphinx"),
+                    Preferences.getEditor("PreviewMarkdownNLtoBR"),
+                    Preferences.getEditor("PreviewMarkdownHTMLFormat"),
+                    Preferences.getEditor("PreviewRestDocutilsHTMLFormat"))
 
-    def __setHtml(self, filePath, html):
+    def __setHtml(self, filePath, html, rootPath):
         """
         Private method to set the HTML to the view and restore the scroll bars
         positions.
         
-        @param filePath file path of the previewed editor (string)
-        @param html processed HTML text ready to be shown (string)
+        @param filePath file path of the previewed editor
+        @type str
+        @param html processed HTML text ready to be shown
+        @type str
+        @param rootPath path of the web site root
+        @type str
         """
         self.__previewedPath = Utilities.normcasepath(
             Utilities.fromNativeSeparators(filePath))
@@ -213,7 +231,11 @@ class PreviewerHTML(QWidget):
                 self.__restoreScrollBarPositions)
             if not filePath:
                 filePath = "/"
-        self.previewView.setHtml(html, baseUrl=QUrl.fromLocalFile(filePath))
+        if rootPath:
+            baseUrl = QUrl.fromLocalFile(rootPath + "/index.html")
+        else:
+            baseUrl = QUrl.fromLocalFile(filePath)
+        self.previewView.setHtml(html, baseUrl=baseUrl)
         if self.__previewedEditor:
             self.__previewedEditor.setFocus()
     
@@ -246,21 +268,24 @@ class PreviewerHTML(QWidget):
                 frame.scrollBarMaximum(Qt.Vertical) == pos.y()
         else:
             from PyQt5.QtCore import QPoint
-            pos = self.__execJavaScript(
-                "(function() {"
-                "var res = {"
-                "    x: 0,"
-                "    y: 0,"
-                "};"
-                "res.x = window.scrollX;"
-                "res.y = window.scrollY;"
-                "return res;"
-                "})()"
-            )
-            if pos is not None:
-                pos = QPoint(pos["x"], pos["y"])
-            else:
-                pos = QPoint(0, 0)
+            try:
+                pos = self.previewView.scrollPosition()
+            except AttributeError:
+                pos = self.__execJavaScript(
+                    "(function() {"
+                    "var res = {"
+                    "    x: 0,"
+                    "    y: 0,"
+                    "};"
+                    "res.x = window.scrollX;"
+                    "res.y = window.scrollY;"
+                    "return res;"
+                    "})()"
+                )
+                if pos is not None:
+                    pos = QPoint(pos["x"], pos["y"])
+                else:
+                    pos = QPoint(0, 0)
             self.__scrollBarPositions[self.__previewedPath] = pos
             self.__hScrollBarAtEnd[self.__previewedPath] = False
             self.__vScrollBarAtEnd[self.__previewedPath] = False
@@ -338,10 +363,11 @@ class PreviewProcessingThread(QThread):
     Class implementing a thread to process some text into HTML usable by the
     previewer view.
     
-    @signal htmlReady(str,str) emitted with the file name and processed HTML
-        to signal the availability of the processed HTML
+    @signal htmlReady(str, str, str) emitted with the file name, the processed
+        HTML and the web site root path to signal the availability of the
+        processed HTML
     """
-    htmlReady = pyqtSignal(str, str)
+    htmlReady = pyqtSignal(str, str, str)
     
     def __init__(self, parent=None):
         """
@@ -354,7 +380,8 @@ class PreviewProcessingThread(QThread):
         self.__lock = threading.Lock()
     
     def process(self, filePath, language, text, ssiEnabled, rootPath,
-                useSphinx):
+                useSphinx, convertNewLineToBreak, markdownHtmlFormat,
+                restDocutilsHtmlFormat):
         """
         Public method to convert the given text to HTML.
         
@@ -366,6 +393,12 @@ class PreviewProcessingThread(QThread):
         @param rootPath root path to be used for SSI processing (str)
         @param useSphinx flag indicating to use Sphinx to generate the
             ReST preview (boolean)
+        @param convertNewLineToBreak flag indicating to convert new lines
+            to HTML break (Markdown only) (boolean)
+        @param markdownHtmlFormat HTML format to be generated by markdown
+            (string)
+        @param restDocutilsHtmlFormat HTML format to be generated by docutils
+            (string)
         """
         with self.__lock:
             self.__filePath = filePath
@@ -375,6 +408,9 @@ class PreviewProcessingThread(QThread):
             self.__rootPath = rootPath
             self.__haveData = True
             self.__useSphinx = useSphinx
+            self.__convertNewLineToBreak = convertNewLineToBreak
+            self.__markdownHtmlFormat = markdownHtmlFormat
+            self.__restDocutilsHtmlFormat = restDocutilsHtmlFormat
             if not self.isRunning():
                 self.start(QThread.LowPriority)
     
@@ -391,19 +427,25 @@ class PreviewProcessingThread(QThread):
                 ssiEnabled = self.__ssiEnabled
                 rootPath = self.__rootPath
                 useSphinx = self.__useSphinx
-                self.__haveData = False
+                convertNewLineToBreak = self.__convertNewLineToBreak
+                markdownHtmlFormat = self.__markdownHtmlFormat
+                restDocutilsHtmlFormat = self.__restDocutilsHtmlFormat
             
+                self.__haveData = False
+
             html = self.__getHtml(language, text, ssiEnabled, filePath,
-                                  rootPath, useSphinx)
+                                  rootPath, useSphinx, convertNewLineToBreak,
+                                  markdownHtmlFormat, restDocutilsHtmlFormat)
             
             with self.__lock:
                 if not self.__haveData:
-                    self.htmlReady.emit(filePath, html)
+                    self.htmlReady.emit(filePath, html, rootPath)
                     break
                 # else - next iteration
     
     def __getHtml(self, language, text, ssiEnabled, filePath, rootPath,
-                  useSphinx):
+                  useSphinx, convertNewLineToBreak, markdownHtmlFormat,
+                  restDocutilsHtmlFormat):
         """
         Private method to process the given text depending upon the given
         language.
@@ -416,17 +458,25 @@ class PreviewProcessingThread(QThread):
         @param rootPath root path to be used for SSI processing (str)
         @param useSphinx flag indicating to use Sphinx to generate the
             ReST preview (boolean)
+        @param convertNewLineToBreak flag indicating to convert new lines
+            to HTML break (Markdown only) (boolean)
+        @param markdownHtmlFormat HTML format to be generated by markdown
+            (string)
+        @param restDocutilsHtmlFormat HTML format to be generated by docutils
+            (string)
         @return processed HTML text (string)
         """
         if language == "HTML":
             if ssiEnabled:
-                return self.__processSSI(text, filePath, rootPath)
+                html = self.__processSSI(text, filePath, rootPath)
             else:
-                return text
+                html = text
+            return self.__processRootPath(html, rootPath)
         elif language == "Markdown":
-            return self.__convertMarkdown(text)
+            return self.__convertMarkdown(text, convertNewLineToBreak,
+                                          markdownHtmlFormat)
         elif language == "ReST":
-            return self.__convertReST(text, useSphinx)
+            return self.__convertReST(text, useSphinx, restDocutilsHtmlFormat)
         else:
             return self.tr(
                 "<p>No preview available for this type of file.</p>")
@@ -478,19 +528,55 @@ class PreviewProcessingThread(QThread):
         
         return txt
     
-    def __convertReST(self, text, useSphinx):
+    def __processRootPath(self, txt, root):
+        """
+        Private method to adjust absolute references to the given root path.
+        
+        @param txt text to be processed
+        @type str
+        @param root directory of the document root
+        @type str
+        @return processed HTML
+        @rtype str
+        """
+        if not root:
+            return txt
+        
+        root = Utilities.fromNativeSeparators(root)
+        if not root.endswith("/"):
+            root += "/"
+        rootLen = len(root)
+        
+        refRe = re.compile(
+            r"""(href|src)=[\\"']/([^\\"']+)[\\"']""",
+            re.IGNORECASE)
+        pos = 0
+        while True:
+            refMatch = refRe.search(txt, pos)
+            if refMatch is None:
+                break
+            
+            txt = (txt[:refMatch.start(0)] + refMatch.group(1) + '="' + root +
+                   refMatch.group(2) + '"' + txt[refMatch.end(0):])
+            pos = refMatch.end(0) + rootLen
+        
+        return txt
+    
+    def __convertReST(self, text, useSphinx, restDocutilsHtmlFormat):
         """
         Private method to convert ReST text into HTML.
         
         @param text text to be processed (string)
         @param useSphinx flag indicating to use Sphinx to generate the
             ReST preview (boolean)
+        @param restDocutilsHtmlFormat HTML format to be generated by docutils
+            (string)
         @return processed HTML (string)
         """
         if useSphinx:
             return self.__convertReSTSphinx(text)
         else:
-            return self.__convertReSTDocutils(text)
+            return self.__convertReSTDocutils(text, restDocutilsHtmlFormat)
     
     def __convertReSTSphinx(self, text):
         """
@@ -540,11 +626,12 @@ class PreviewProcessingThread(QThread):
         
         return html
     
-    def __convertReSTDocutils(self, text):
+    def __convertReSTDocutils(self, text, htmlFormat):
         """
         Private method to convert ReST text into HTML using 'docutils'.
         
         @param text text to be processed (string)
+        @param htmlFormat HTML format to be generated (string)
         @return processed HTML (string)
         """
         if 'sphinx' in sys.modules:
@@ -567,35 +654,42 @@ class PreviewProcessingThread(QThread):
         # redirect sys.stderr because we are not interested in it here
         origStderr = sys.stderr
         sys.stderr = io.StringIO()
-        html = docutils.core.publish_string(text, writer_name='html')\
-            .decode("utf-8")
+        html = docutils.core.publish_string(
+            text, writer_name=htmlFormat.lower()).decode("utf-8")
         sys.stderr = origStderr
         return html
     
-    def __convertMarkdown(self, text):
+    def __convertMarkdown(self, text, convertNewLineToBreak, htmlFormat):
         """
         Private method to convert Markdown text into HTML.
         
         @param text text to be processed (string)
+        @param convertNewLineToBreak flag indicating to convert new lines
+            to HTML break (Markdown only) (boolean)
+        @param htmlFormat HTML format to be generated by markdown (string)
         @return processed HTML (string)
         """
         try:
             import markdown     # __IGNORE_EXCEPTION__
         except ImportError:
             return self.tr(
-                """<p>Markdown preview requires the <b>python-markdown</b> """
+                """<p>Markdown preview requires the <b>Markdown</b> """
                 """package.<br/>Install it with your package manager,"""
-                """ 'pip install docutils' or see """
+                """ 'pip install Markdown' or see """
                 """<a href="http://pythonhosted.org/Markdown/install.html">"""
                 """installation instructions.</a></p>""")
         
         try:
             import mdx_mathjax  # __IGNORE_EXCEPTION__ __IGNORE_WARNING__
         except ImportError:
-            #mathjax doesn't require import statement if installed as extension
+            # mathjax doesn't require import statement if installed
+            # as extension
             pass
-
-        extensions = ['fenced_code', 'nl2br', 'extra']
+        
+        if convertNewLineToBreak:
+            extensions = ['fenced_code', 'nl2br', 'extra']
+        else:
+            extensions = ['fenced_code', 'extra']
         
         # version 2.0 supports only extension names, not instances
         if markdown.version_info[0] > 2 or \
@@ -606,9 +700,8 @@ class PreviewProcessingThread(QThread):
                 Class is placed here, because it depends on imported markdown,
                 and markdown import is lazy.
                 
-                (see http://achinghead.com/
-                python-markdown-adding-insert-delete.html this page for
-                details)
+                (see https://pythonhosted.org/Markdown/extensions/api.html
+                this page for details)
                 """
                 DEL_RE = r'(~~)(.*?)~~'
 
@@ -622,9 +715,11 @@ class PreviewProcessingThread(QThread):
             extensions.append(_StrikeThroughExtension())
 
         try:
-            return markdown.markdown(text, extensions + ['mathjax'])
+            return markdown.markdown(text, extensions=extensions + ['mathjax'],
+                                     output_format=htmlFormat.lower())
         except (ImportError, ValueError):
             # markdown raises ValueError or ImportError, depends on version
             # It is not clear, how to distinguish missing mathjax from other
             # errors. So keep going without mathjax.
-            return markdown.markdown(text, extensions)
+            return markdown.markdown(text, extensions=extensions,
+                                     output_format=htmlFormat.lower())
