@@ -1052,13 +1052,13 @@ def createConfig():
     apis = []
     if installApis:
         for progLanguage in progLanguages:
-            for apiName in glob.glob(
-                    os.path.join(sourceDir, "APIs", progLanguage, "*.api")):
+            for apiName in sorted(glob.glob(
+                    os.path.join(sourceDir, "APIs", progLanguage, "*.api"))):
                 apis.append(os.path.basename(apiName))
             if progLanguage == "Python":
                 # treat Python3 API files the same as Python API files
-                for apiName in glob.glob(
-                        os.path.join(sourceDir, "APIs", "Python3", "*.api")):
+                for apiName in sorted(glob.glob(
+                        os.path.join(sourceDir, "APIs", "Python3", "*.api"))):
                     apis.append(os.path.basename(apiName))
     
     fn = 'eric6config.py'
